# -*- coding: utf-8 -*-
"""
Created on Mon Sep 20 14:55:40 2021

@author: xavier.mouy
"""
import os
import sys
sys.path.append(r'C:\Users\xavier.mouy\Documents\GitHub\ecosound') # Adds higher directory to python modules path.

import pandas as pd
import matplotlib.pyplot as plt
import mpl_toolkits.mplot3d
import matplotlib.cm
import matplotlib as mpl
from mpl_toolkits.axes_grid1 import make_axes_locatable
#import scipy.spatial
import numpy as np
import datetime
import math
import cv2

from ecosound.core.audiotools import Sound, upsample
from ecosound.core.spectrogram import Spectrogram
from ecosound.core.measurement import Measurement
from ecosound.detection.detector_builder import DetectorFactory
from ecosound.visualization.grapher_builder import GrapherFactory
import ecosound.core.tools
from ecosound.core.tools import derivative_1d, envelope, read_yaml
from localizationlib import euclidean_dist, calc_hydrophones_distances, calc_tdoa, defineReceiverPairs, defineJacobian, predict_tdoa, linearized_inversion, solve_iterative_ML, defineCubeVolumeGrid, defineSphereVolumeGrid
import platform
import cv2

def plot_spectrogram(audio_file,loc,t1_sec, t2_sec, geometry=(1,1,1)):
    
    fmin=0
    fmax=1000
    frame= 0.0625
    window_type= 'hann'
    nfft=0.0853
    step=0.01
    channel=0
    chunk=[t1_sec,t2_sec]
    
    graph_spectros = GrapherFactory('SoundPlotter', title='Spectrograms', frequency_max=fmax)
    sound = Sound(audio_file)
    sound.read(channel=channel, chunk=chunk, unit='sec', detrend=True)
    # Calculates  spectrogram
    spectro = Spectrogram(frame, window_type, nfft, step, sound.waveform_sampling_frequency, unit='sec')
    spectro.compute(sound, dB=True, use_dask=False)
    # Crop unused frequencies
    spectro.crop(frequency_min=fmin, frequency_max=fmax, inplace=True)
    # Plot
    graph_spectros.add_data(spectro)    
    
    #graph_spectros.add_annotation(loc, panel=0, color='burlywood',label='Detections')
    graph_spectros.add_annotation(loc, panel=0, color='peachpuff')
    
    graph_spectros.colormap = 'binary' #'jet'
    fig, ax = graph_spectros.show()

    if ax.get_geometry() != geometry :
        ax.change_geometry(*geometry)        
    return fig, ax

def plot_top_view(hydrophones_config,loc_data,params,cmap,norm, ax):
    
    #fig1 = plt.figure()
    #ax = fig1.add_subplot(111)
    colors = matplotlib.cm.tab10(hydrophones_config.index.values)
    
    # plot hydrophones
    for index, hp in hydrophones_config.iterrows():
        point = ax.scatter(hp['x'],hp['y'],
                           s=20,
                           #color=colors[index],
                           color='gainsboro',
                           edgecolors='dimgray',
                           #markeredgecolor='r',
                           label=hp['name'],
                           alpha=1,
                           zorder=3
                           )

    # plot frame
    frame_color = 'whitesmoke'
    frame_alpha = 1
    frame_width = 3
    
    rectangle = plt.Rectangle((-0.94,-0.94), 1.88, 1.9,
                              linewidth=frame_width,
                              ec='mistyrose',
                              alpha=frame_alpha,
                              facecolor='none')
    
    ax.add_patch(rectangle)
    
    rectangle2 = plt.Rectangle((-0.15,-0.94), 0.178, 1.9,
                              linewidth=frame_width,
                              ec='mistyrose',
                              alpha=frame_alpha,
                              facecolor='mistyrose')
    ax.add_patch(rectangle2)
    # ax.plot([0.028, 0.028],[-0.94, 0.96],
    #      linewidth=frame_width ,
    #      alpha=frame_alpha,
    #      linestyle= 'solid',
    #      color='mistyrose',         
    #      )
    # ax.plot([-0.15, -0.15],[-0.94, 0.96],
    #      linewidth=frame_width ,
    #      alpha=frame_alpha,
    #      linestyle= 'solid',
    #      color='mistyrose',
    #      )
    
    #c=z, cmap=cmap, norm=norm
    
    # plot fishcam 02
    rectangle4 = plt.Rectangle((1.3,-0.1), 0.4, 0.2,linewidth=frame_width,ec=frame_color,alpha=frame_alpha, facecolor=frame_color)
    ax.add_patch(rectangle4)
    
    ax.plot([0.93, 2],[0.5, 0.5],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )
    ax.plot([0.93, 2],[-0.5, -0.5],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )
    ax.plot([1.5, 1.5],[0.5, -0.5],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )
    
    
    # plot localizations
    ax.scatter(loc_data['x'], loc_data['y'],c=loc_data['time_min_offset'],
                        s=params['loc_size'].values[0],
                        marker=params['loc_marker'].values[0],
                        #color=params['loc_color'].values[0],
                        alpha=params['loc_alpha'].values[0],
                        cmap=cmap,
                        norm=norm,
                        zorder=5,
                        )
    # plot uncertainties
    for idx, loc_point in loc_data.iterrows():   
        ax.plot([loc_point['x']-loc_point['x_std'],loc_point['x']+loc_point['x_std']],
                [loc_point['y'],loc_point['y']],
                #c=loc_point['time_min_offset'],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                #color=params['uncertainty_color'].values[0],
                color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                #cmap=cmap,
                #norm=norm,
                )
    
        ax.plot([loc_point['x'],loc_point['x']],
                [loc_point['y']-loc_point['y_std'],loc_point['y']+loc_point['y_std']],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                #color=params['uncertainty_color'].values[0],
                color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                )
        
    # Axes labels
    ax.set_xlabel('X (m)', labelpad=4)
    ax.set_ylabel('Y (m)', labelpad=4)
    ax.set_xlim(params['x_min'].values[0], params['x_max'].values[0])
    ax.set_ylim(params['y_min'].values[0], params['y_max'].values[0])
    #ax.grid()
    ax.set_aspect('equal', adjustable='box')
    return ax


def plot_side_view(hydrophones_config,loc_data,params,cmap,norm, ax):
    
    #fig1 = plt.figure()
    #ax = fig1.add_subplot(111)
    colors = matplotlib.cm.tab10(hydrophones_config.index.values)
    
    # plot hydrophones
    for index, hp in hydrophones_config.iterrows():
        point = ax.scatter(hp['x'],hp['z'],
                           s=20,
                           #color=colors[index],
                           color='gainsboro',
                           edgecolors='dimgray',
                           #markeredgecolor='r',
                           label=hp['name'],
                           alpha=1,
                           zorder=3
                           )

    # plot frame
    frame_color = 'whitesmoke'
    frame_alpha = 1
    frame_width = 3
    
    rectangle1 = plt.Rectangle((-0.94,-0.76), 1.87, 1.5,linewidth=frame_width,ec=frame_color,alpha=frame_alpha, facecolor='none')
    rectangle2 = plt.Rectangle((-0.94, 0.74), 1.87, 1,linewidth=frame_width,ec=frame_color,alpha=frame_alpha, facecolor='none')
    rectangle3 = plt.Rectangle((-0.1, 1.55), 0.2, 0.4,linewidth=frame_width,ec=frame_color,alpha=frame_alpha, facecolor=frame_color)
    ax.add_patch(rectangle1)
    ax.add_patch(rectangle2)
    ax.add_patch(rectangle3)
    ax.plot([-0.94, -0.94],[-0.76, -1.06],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )
    ax.plot([0.93, 0.93],[-0.76, -1.06],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )
    ax.plot([0.028, 0.028],[0, -0.76],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )
    
    ax.plot([-0.94, 0.93],[-0.76, -0.76],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color='mistyrose',
         )
    
    # #field of view
    # ax.plot([-0.10, -1.2],[1.55, 0.75],
    #       linewidth=1 ,
    #       alpha=0.4,
    #       linestyle= 'solid',
    #       color='red',
    #       )
    # ax.plot([0.10, 1.2],[1.55, 0.75],
    #       linewidth=1 ,
    #       alpha=0.4,
    #       linestyle= 'solid',
    #       color='red',
    #       )    
    
    # plot fishcam 02
    rectangle4 = plt.Rectangle((1.3, -0.22), 0.4, 0.2,linewidth=frame_width,ec=frame_color,alpha=frame_alpha, facecolor=frame_color)
    ax.add_patch(rectangle4)
    ax.plot([0.93, 1.5],[-0.76, -0.2],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )
    ax.plot([1.5, 2],[-0.1, -1.06],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )    
    
    # plot localizations
    ax.scatter(loc_data['x'], loc_data['z'],c=loc_data['time_min_offset'],
                        s=params['loc_size'].values[0],
                        marker=params['loc_marker'].values[0],
                        #color=params['loc_color'].values[0],
                        alpha=params['loc_alpha'].values[0],
                        cmap=cmap,
                        norm=norm,
                        zorder=5
                        )
    # plot uncertainties
    for idx, loc_point in loc_data.iterrows():   
        ax.plot([loc_point['x']-loc_point['x_std'],loc_point['x']+loc_point['x_std']],
                [loc_point['z'],loc_point['z']],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                #color=params['uncertainty_color'].values[0],
                color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                )
    
        ax.plot([loc_point['x'],loc_point['x']],
                [loc_point['z']-loc_point['z_std'],loc_point['z']+loc_point['z_std']],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                #color=params['uncertainty_color'].values[0],
                color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                )
        
    # Axes labels
    ax.set_xlabel('X (m)', labelpad=4)
    ax.set_ylabel('Z (m)', labelpad=4)
    ax.set_xlim(params['x_min'].values[0], params['x_max'].values[0])
    ax.set_ylim(params['z_min'].values[0], params['z_max'].values[0])
    #ax.grid()
    ax.set_aspect('equal', adjustable='box')
    return ax

def plot_video_frame(video_file,frame_time_sec, ax):
    frame = []
    cap = cv2.VideoCapture(video_file)
    if (cap.isOpened()== False):        
        print("Error opening video stream or file")
    else:
     	# Read until video is completed
        idx=0
        while(cap.isOpened()):            
            ret, frame = cap.read()            
            if ret == True:
                idx+=1
                fps = cap.get(cv2.CAP_PROP_FPS)  # OpenCV2 version 2 used "CV_CAP_PROP_FPS"                
                current_time = idx*(1/fps)
                if current_time >= frame_time_sec:
                    frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB) 
                    frame = frame[90:900,340:1220]
                    break                
            # Break the loop
            else:
                break
        cap.release()
        #cv2.destroyAllWindows()
    return ax.imshow(frame)
## ###########################################################################

def plot_full_figure(time_sec=None):
    
    # loc_file1 = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\results\large-array_lingcod2\locs_part1.nc'
    # loc_file2 = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\results\large-array_lingcod2\locs_part2.nc'
    # audio_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\AMAR173.4.20190916T204248Z.wav'
    # video_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\3420_FishCam01_20190920T163627.613206Z_1600x1200_awb-auto_exp-night_fr-10_q-20_sh-0_b-50_c-0_i-400_sat-0.mp4'
    # hp_config_file =  r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\hydrophones_config_07-HI.csv'
    # t1_sec = 589
    # t2_sec = 635#604
    
    loc_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\results\large-array_lingcod3\locs.nc'
    audio_file = infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\AMAR173.4.20190617T151307Z.wav'
    video_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\3420_FishCam01_20190920T163627.613206Z_1600x1200_awb-auto_exp-night_fr-10_q-20_sh-0_b-50_c-0_i-400_sat-0.mp4'
    hp_config_file =  r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\hydrophones_config_04-OGD.csv'
    t1_sec = 217
    t2_sec = 227


    filter_x=[-2.5, 2]
    filter_y=[-1, 1]
    filter_z=[-2.1, 2.5]
    filter_x_std=0.3
    filter_y_std=0.3
    filter_z_std=0.3
    
    params=pd.DataFrame({
        'loc_color': ['black'],
        'loc_marker': ['o'],
        'loc_alpha': [1],
        'loc_size': [5],
        'uncertainty_color': ['black'],
        'uncertainty_style': ['-'],
        'uncertainty_alpha': [1], #0.7
        'uncertainty_width': [0.2], #0.2
        'x_min':[-2.5],
        'x_max':[2],
        'y_min':[-1.6],
        'y_max':[1.5],
        'z_min':[-1.5],
        'z_max':[2],    
        })
        
    ## ###########################################################################
    
    ## load localization results
    loc = Measurement()
    loc.from_netcdf(loc_file)
    loc_data = loc.data    
    
    # loc = Measurement()
    # loc.from_netcdf(loc_file1)
    # loc_data = loc.data    
    # loc2 = Measurement()
    # loc2.from_netcdf(loc_file2)
    # loc_data2 = loc2.data    
    # loc_data = loc_data.append(loc_data2, ignore_index = True)
    
    ## load hydrophone locations
    hydrophones_config = pd.read_csv(hp_config_file)
    
    # Filter
    loc_data = loc_data.dropna(subset=['x', 'y','z']) # remove NaN
    loc_data = loc_data.loc[(loc_data['x']>=min(filter_x)) & 
                            (loc_data['x']<=max(filter_x)) &
                            (loc_data['y']>=min(filter_y)) & 
                            (loc_data['y']<=max(filter_y)) &
                            (loc_data['z']>=min(filter_z)) & 
                            (loc_data['z']<=max(filter_z)) &
                            (loc_data['x_std']<= filter_x_std) & 
                            (loc_data['y_std']<= filter_y_std) &
                            (loc_data['z_std']<= filter_z_std)
                            ]
    # Adjust detection times
    loc_data['time_min_offset'] = loc_data['time_min_offset'] - t1_sec
    loc_data['time_max_offset'] = loc_data['time_max_offset'] - t1_sec
    
    if time_sec!= None:
        loc_data = loc_data.loc[(loc_data['time_max_offset']<=time_sec)] 
    else:
        print('Static')
    
    # update loc object
    loc.data = loc_data
    
    # plots
    # fig, ax = plt.subplots(figsize=(6, 1))
    # fig.subplots_adjust(bottom=0.5)
    # n_colors = t2_sec-t1_sec
    # cmap = mpl.cm.get_cmap('CMRmap', n_colors*2)
    # norm = mpl.colors.Normalize(vmin=0, vmax=n_colors)
    # ax_cmap = mpl.colorbar.ColorbarBase(ax, cmap=cmap,
    #                                 norm=norm,
    #                                 orientation='horizontal')
    # ax_cmap.set_label('Time (s)')
    
   
    
    # Plot spectrogram
    fig_final, ax_spectro = plot_spectrogram(audio_file,loc,t1_sec, t2_sec, geometry=(5,1,1))    
    ax_spectro.set_title("")
    ax_spectro.get_xaxis().set_visible(False)
    n_colors = t2_sec-t1_sec
    cmap = mpl.cm.get_cmap('viridis', n_colors*4)
    norm = mpl.colors.Normalize(vmin=0, vmax=n_colors)
    divider = make_axes_locatable(ax_spectro)
    cax = divider.append_axes('bottom', 0.1, pad=0.03 )
    ax_cmap = mpl.colorbar.ColorbarBase(cax, cmap=cmap,
                                    norm=norm,
                                    orientation='horizontal')
    ax_cmap.set_label('Time (s)')
    
    if time_sec:
        SFreq_min, SFreq_max = ax_spectro.get_ylim()
        ax_spectro.plot([time_sec,time_sec],[SFreq_min,SFreq_max],'r')
    
    # plot detection points on top of spectrogram
    #gs0 = fig_final.add_gridspec(60,1)
    ax_detec = fig_final.add_subplot(20,1,1)
    det_y = np.asarray(np.ones((1,len(loc_data['time_min_offset']))))[0]
    det_x = np.asarray(loc_data['time_min_offset'])
    ax_detec.scatter(det_x,det_y, c=loc_data['time_min_offset'],cmap=cmap,norm=norm,s=12)
    ax_detec.set_xlim(ax_spectro.get_xlim())
    ax_detec.get_xaxis().set_visible(False)
    ax_detec.get_yaxis().set_visible(False)
    ax_detec.axis('off')
    
    # #pos =[left, bottom, width, height]
    # box = ax_detec.get_position()
    # box.y0 = box.y0 + 0.6
    # box.y1 = box.y1 + 0.6
    # ax_detec.set_position(box)
    
    #size = fig_final.get_size_inches()
    
    
    plt.subplots_adjust(left=0.08, bottom=0.1, right=0.95, top=0.95, wspace=0, hspace=0)
    
    
    # divider2 = make_axes_locatable(ax_spectro)
    # cax2 = divider2.append_axes('top', size=0.2, pad=10.0)
    # det_y = np.asarray(np.ones((1,len(loc_data['time_min_offset']))))[0]
    # det_x = np.asarray(loc_data['time_min_offset'])
    # cax2.plot(det_x,det_y,'.r')
    # cax2.set_xlim(ax_spectro.get_xlim())
    
    
    # ax_cmap = mpl.colorbar.ColorbarBase(cax, cmap=cmap,
    #                                     norm=norm,
    #                                     orientation='horizontal')
    
    
    
    gs = fig_final.add_gridspec(3,2)
    
    # plot localization top
    ax_toploc = fig_final.add_subplot(gs[1:,1])
    plot_top_view(hydrophones_config,loc_data,params,cmap,norm, ax_toploc)
    ax_toploc.set_anchor('E')
    
    # plot localization side
    #ax_sideloc = fig_final.add_subplot(3,3,7,sharex = ax_toploc)
    ax_sideloc = fig_final.add_subplot(gs[1:,0])
    plot_side_view(hydrophones_config,loc_data,params,cmap,norm,ax_sideloc)
    ax_sideloc.set_anchor('W')
    
    # set the spacing between subplots
    plt.subplots_adjust(wspace=0, hspace=0)
    
    
    # # plot video frame 1
    # fig_video1, ax_video1 = plt.subplots(1,1)
    # frame1_sec = 152.8 # second detection -> 16:38:59.8
    # #ax_video1 = fig_final.add_subplot(3,3,5)
    # plot_video_frame(video_file,frame1_sec, ax_video1)
    # ax_video1.get_xaxis().set_visible(False)
    # ax_video1.get_yaxis().set_visible(False)
    
    # # plot video frame 2
    # fig_video2, ax_video2 = plt.subplots(1,1)
    # frame2_sec = 160 # 4th detection -> 16:39:07
    # #ax_video2 = fig_final.add_subplot(3,3,6)
    # plot_video_frame(video_file,frame2_sec, ax_video2)
    # ax_video2.get_xaxis().set_visible(False)
    # ax_video2.get_yaxis().set_visible(False)
    
    
    fig_final.set_size_inches(13.66,  6.81)

    box = ax_spectro.get_position()
    box.y0 = box.y0 - 0.03
    box.y1 = box.y1 - 0.03
    ax_spectro.set_position(box)
    return fig_final


def main():
    
    # # # static
    # fig = plot_full_figure()
    # #print('s')
    
    # movie
    outdir = r'C:\Users\xavier.mouy\Documents\PhD\Thesis\phd-thesis\Figures\XAV_arrays\LargeArray_lingcod\animation'
    fps=20
    duration_sec = 10
    
    # create individual frames
    times = np.arange(0,duration_sec,1/fps)
    for idx, t in enumerate(times):
        plot_full_figure(time_sec=t)
        plt.savefig(os.path.join(outdir,str(idx)+'.jpg'))
        plt.close()
    
    # stack all frames
    img_array = []
    for filename in range(0,idx):
        img = cv2.imread(os.path.join(outdir,str(filename)+'.jpg'))
        height, width, layers = img.shape
        size = (width,height)
        img_array.append(img)
    # write videos
    out = cv2.VideoWriter(os.path.join(outdir,'animation.mp4'),cv2.VideoWriter_fourcc(*'DIVX'), fps, size) 
    for i in range(len(img_array)):
        out.write(img_array[i])
    out.release()
    
if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
"""
Created on Mon Oct 11 09:27:05 2021

@author: xavier.mouy
"""
import os
import sys
sys.path.append(r'C:\Users\xavier.mouy\Documents\GitHub\ecosound') # Adds higher directory to python modules path.

import pandas as pd
from ecosound.core.measurement import Measurement

indir = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\results\mini-array_copper'

# load data
print('')
print('Loading dataset')
idx = 0
for infile in os.listdir(indir):
    if infile.endswith(".nc"):
        print(infile)
        
        locs = Measurement()
        locs.from_netcdf(os.path.join(indir,infile))
        loc_data = locs.data
        if idx == 0:
            final_df = loc_data
        else:
            final_df = final_df.append(loc_data,ignore_index=True)        
        idx +=1

final_df.to_csv(os.path.join(indir,'localizations_python.csv'))


# -*- coding: utf-8 -*-
"""
Created on Mon Sep 20 14:55:40 2021

@author: xavier.mouy
"""
import os
import sys
sys.path.append(r'C:\Users\xavier.mouy\Documents\GitHub\ecosound') # Adds higher directory to python modules path.

import pandas as pd
import matplotlib.pyplot as plt
import mpl_toolkits.mplot3d
import matplotlib.cm
import matplotlib as mpl
from mpl_toolkits.axes_grid1 import make_axes_locatable
#import scipy.spatial
import numpy as np
import datetime
import math

from ecosound.core.audiotools import Sound, upsample
from ecosound.core.spectrogram import Spectrogram
from ecosound.core.measurement import Measurement
from ecosound.detection.detector_builder import DetectorFactory
from ecosound.visualization.grapher_builder import GrapherFactory
import ecosound.core.tools
from ecosound.core.tools import derivative_1d, envelope, read_yaml
from localizationlib import euclidean_dist, calc_hydrophones_distances, calc_tdoa, defineReceiverPairs, defineJacobian, predict_tdoa, linearized_inversion, solve_iterative_ML, defineCubeVolumeGrid, defineSphereVolumeGrid
import platform
import cv2

def plot_spectrogram(audio_file,loc,t1_sec, t2_sec, geometry=(1,1,1)):
    
    fmin=0
    fmax=1000
    frame= 0.0625
    window_type= 'hann'
    nfft=0.0853
    step=0.01
    channel=0
    chunk=[t1_sec,t2_sec]
    
    graph_spectros = GrapherFactory('SoundPlotter', title='Spectrograms', frequency_max=fmax)
    sound = Sound(audio_file)
    sound.read(channel=channel, chunk=chunk, unit='sec', detrend=True)
    # Calculates  spectrogram
    spectro = Spectrogram(frame, window_type, nfft, step, sound.waveform_sampling_frequency, unit='sec')
    spectro.compute(sound, dB=True, use_dask=False)
    # Crop unused frequencies
    spectro.crop(frequency_min=fmin, frequency_max=fmax, inplace=True)
    # Plot
    graph_spectros.add_data(spectro)    
    
    #graph_spectros.add_annotation(loc, panel=0, color='burlywood',label='Detections')
    graph_spectros.add_annotation(loc, panel=0, color='peachpuff')
    
    graph_spectros.colormap = 'binary' #'jet'
    fig, ax = graph_spectros.show()

    if ax.get_geometry() != geometry :
        ax.change_geometry(*geometry)        
    return fig, ax

def plot_top_view(hydrophones_config,loc_data,params,cmap,norm, ax):
    
    #fig1 = plt.figure()
    #ax = fig1.add_subplot(111)
    colors = matplotlib.cm.tab10(hydrophones_config.index.values)
    
    # plot hydrophones
    for index, hp in hydrophones_config.iterrows():
        point = ax.scatter(hp['x'],hp['y'],
                           s=20,
                           #color=colors[index],
                           color='gainsboro',
                           edgecolors='dimgray',
                           #markeredgecolor='r',
                           label=hp['name'],
                           alpha=1,
                           zorder=3
                           )

    # plot frame
    frame_color = 'whitesmoke'
    frame_alpha = 1
    frame_width = 3
    
    rectangle = plt.Rectangle((-0.94,-0.94), 1.88, 1.9,
                              linewidth=frame_width,
                              ec='mistyrose',
                              alpha=frame_alpha,
                              facecolor='none')
    
    ax.add_patch(rectangle)
    
    rectangle2 = plt.Rectangle((-0.15,-0.94), 0.178, 1.9,
                              linewidth=frame_width,
                              ec='mistyrose',
                              alpha=frame_alpha,
                              facecolor='mistyrose')
    ax.add_patch(rectangle2)
    # ax.plot([0.028, 0.028],[-0.94, 0.96],
    #      linewidth=frame_width ,
    #      alpha=frame_alpha,
    #      linestyle= 'solid',
    #      color='mistyrose',         
    #      )
    # ax.plot([-0.15, -0.15],[-0.94, 0.96],
    #      linewidth=frame_width ,
    #      alpha=frame_alpha,
    #      linestyle= 'solid',
    #      color='mistyrose',
    #      )
    
    #c=z, cmap=cmap, norm=norm
    # plot localizations
    ax.scatter(loc_data['x'], loc_data['y'],c=loc_data['time_min_offset'],
                        s=params['loc_size'].values[0],
                        marker=params['loc_marker'].values[0],
                        #color=params['loc_color'].values[0],
                        alpha=params['loc_alpha'].values[0],
                        cmap=cmap,
                        norm=norm,
                        zorder=5,
                        )
    # plot uncertainties
    for idx, loc_point in loc_data.iterrows():   
        ax.plot([loc_point['x']-loc_point['x_std'],loc_point['x']+loc_point['x_std']],
                [loc_point['y'],loc_point['y']],
                #c=loc_point['time_min_offset'],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                #color=params['uncertainty_color'].values[0],
                color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                #cmap=cmap,
                #norm=norm,
                )
    
        ax.plot([loc_point['x'],loc_point['x']],
                [loc_point['y']-loc_point['y_std'],loc_point['y']+loc_point['y_std']],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                #color=params['uncertainty_color'].values[0],
                color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                )
        
    # Axes labels
    ax.set_xlabel('X (m)', labelpad=4)
    ax.set_ylabel('Y (m)', labelpad=4)
    ax.set_xlim(params['x_min'].values[0], params['x_max'].values[0])
    ax.set_ylim(params['y_min'].values[0], params['y_max'].values[0])
    #ax.grid()
    ax.set_aspect('equal', adjustable='box')
    return ax


def plot_side_view(hydrophones_config,loc_data,params,cmap,norm, ax):
    
    #fig1 = plt.figure()
    #ax = fig1.add_subplot(111)
    colors = matplotlib.cm.tab10(hydrophones_config.index.values)
    
    # plot hydrophones
    for index, hp in hydrophones_config.iterrows():
        point = ax.scatter(hp['x'],hp['z'],
                           s=20,
                           #color=colors[index],
                           color='gainsboro',
                           edgecolors='dimgray',
                           #markeredgecolor='r',
                           label=hp['name'],
                           alpha=1,
                           zorder=3
                           )

    # plot frame
    frame_color = 'whitesmoke'
    frame_alpha = 1
    frame_width = 3
    
    rectangle1 = plt.Rectangle((-0.94,-0.76), 1.87, 1.5,linewidth=frame_width,ec=frame_color,alpha=frame_alpha, facecolor='none')
    rectangle2 = plt.Rectangle((-0.94, 0.74), 1.87, 1,linewidth=frame_width,ec=frame_color,alpha=frame_alpha, facecolor='none')
    rectangle3 = plt.Rectangle((-0.1, 1.55), 0.2, 0.4,linewidth=frame_width,ec=frame_color,alpha=frame_alpha, facecolor=frame_color)
    ax.add_patch(rectangle1)
    ax.add_patch(rectangle2)
    ax.add_patch(rectangle3)
    ax.plot([-0.94, -0.94],[-0.76, -1.06],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )
    ax.plot([0.93, 0.93],[-0.76, -1.06],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )
    ax.plot([0.028, 0.028],[0, -0.76],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color=frame_color,
         )
    
    ax.plot([-0.94, 0.93],[-0.76, -0.76],
         linewidth=frame_width ,
         alpha=frame_alpha,
         linestyle= 'solid',
         color='mistyrose',
         )
    
    # #field of view
    # ax.plot([-0.10, -1.2],[1.55, 0.75],
    #       linewidth=1 ,
    #       alpha=0.4,
    #       linestyle= 'solid',
    #       color='red',
    #       )
    # ax.plot([0.10, 1.2],[1.55, 0.75],
    #       linewidth=1 ,
    #       alpha=0.4,
    #       linestyle= 'solid',
    #       color='red',
    #       )    
    
    # plot localizations
    ax.scatter(loc_data['x'], loc_data['z'],c=loc_data['time_min_offset'],
                        s=params['loc_size'].values[0],
                        marker=params['loc_marker'].values[0],
                        #color=params['loc_color'].values[0],
                        alpha=params['loc_alpha'].values[0],
                        cmap=cmap,
                        norm=norm,
                        zorder=5
                        )
    # plot uncertainties
    for idx, loc_point in loc_data.iterrows():   
        ax.plot([loc_point['x']-loc_point['x_std'],loc_point['x']+loc_point['x_std']],
                [loc_point['z'],loc_point['z']],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                #color=params['uncertainty_color'].values[0],
                color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                )
    
        ax.plot([loc_point['x'],loc_point['x']],
                [loc_point['z']-loc_point['z_std'],loc_point['z']+loc_point['z_std']],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                #color=params['uncertainty_color'].values[0],
                color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                )
        
    # Axes labels
    ax.set_xlabel('X (m)', labelpad=4)
    ax.set_ylabel('Z (m)', labelpad=4)
    ax.set_xlim(params['x_min'].values[0], params['x_max'].values[0])
    ax.set_ylim(params['z_min'].values[0], params['z_max'].values[0])
    #ax.grid()
    ax.set_aspect('equal', adjustable='box')
    return ax

def plot_video_frame(video_file,frame_time_sec, ax):
    frame = []
    cap = cv2.VideoCapture(video_file)
    if (cap.isOpened()== False):        
        print("Error opening video stream or file")
    else:
     	# Read until video is completed
        idx=0
        while(cap.isOpened()):            
            ret, frame = cap.read()            
            if ret == True:
                idx+=1
                fps = cap.get(cv2.CAP_PROP_FPS)  # OpenCV2 version 2 used "CV_CAP_PROP_FPS"                
                current_time = idx*(1/fps)
                if current_time >= frame_time_sec:
                    frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB) 
                    frame = frame[90:900,340:1220]
                    break                
            # Break the loop
            else:
                break
        cap.release()
        #cv2.destroyAllWindows()
    return ax.imshow(frame)
## ###########################################################################

loc_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\results\large-array_lingcod\AMAR173.4.20190919T204248Z.nc'
audio_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\AMAR173.4.20190919T204248Z.wav'
video_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\3284_FishCam01_20190919T205129.843851Z_1600x1200_awb-auto_exp-night_fr-10_q-20_sh-0_b-50_c-0_i-400_sat-0.mp4'
hp_config_file =  r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\hydrophones_config_07-HI.csv'
t1_sec = 697
t2_sec = 700

filter_x=[-5, 5]
filter_y=[-3, 3]
filter_z=[-1.5, 3]
filter_x_std=1
filter_y_std=1
filter_z_std=1

params=pd.DataFrame({
    'loc_color': ['black'],
    'loc_marker': ['o'],
    'loc_alpha': [1],
    'loc_size': [5],
    'uncertainty_color': ['black'],
    'uncertainty_style': ['-'],
    'uncertainty_alpha': [1], #0.7
    'uncertainty_width': [0.2], #0.2
    'x_min':[-3],
    'x_max':[3],
    'y_min':[-1.26],
    'y_max':[1.26],
    'z_min':[-1.5],
    'z_max':[2.1],    
    })
    
## ###########################################################################

## load localization results
loc = Measurement()
loc.from_netcdf(loc_file)
loc_data = loc.data

## load hydrophone locations
hydrophones_config = pd.read_csv(hp_config_file)

# Filter
loc_data = loc_data.dropna(subset=['x', 'y','z']) # remove NaN
loc_data = loc_data.loc[(loc_data['x']>=min(filter_x)) & 
                        (loc_data['x']<=max(filter_x)) &
                        (loc_data['y']>=min(filter_y)) & 
                        (loc_data['y']<=max(filter_y)) &
                        (loc_data['z']>=min(filter_z)) & 
                        (loc_data['z']<=max(filter_z)) &
                        (loc_data['x_std']<= filter_x_std) & 
                        (loc_data['y_std']<= filter_y_std) &
                        (loc_data['z_std']<= filter_z_std)
                        ]
# Adjust detection times
loc_data['time_min_offset'] = loc_data['time_min_offset'] - t1_sec
loc_data['time_max_offset'] = loc_data['time_max_offset'] - t1_sec

# update loc object
loc.data = loc_data

# plots
# fig, ax = plt.subplots(figsize=(6, 1))
# fig.subplots_adjust(bottom=0.5)
# n_colors = t2_sec-t1_sec
# cmap = mpl.cm.get_cmap('CMRmap', n_colors*2)
# norm = mpl.colors.Normalize(vmin=0, vmax=n_colors)
# ax_cmap = mpl.colorbar.ColorbarBase(ax, cmap=cmap,
#                                 norm=norm,
#                                 orientation='horizontal')
# ax_cmap.set_label('Time (s)')

# Plot spectrogram
fig_final, ax_spectro = plot_spectrogram(audio_file,loc,t1_sec, t2_sec, geometry=(5,1,1))    
ax_spectro.set_title("")
ax_spectro.get_xaxis().set_visible(False)
n_colors = t2_sec-t1_sec
cmap = mpl.cm.get_cmap('viridis', n_colors*4)
norm = mpl.colors.Normalize(vmin=0, vmax=n_colors)
divider = make_axes_locatable(ax_spectro)
cax = divider.append_axes('bottom', 0.1, pad=0.03 )
ax_cmap = mpl.colorbar.ColorbarBase(cax, cmap=cmap,
                                norm=norm,
                                orientation='horizontal')
ax_cmap.set_label('Time (s)')



# plot detection points on top of spectrogram
#gs0 = fig_final.add_gridspec(60,1)
ax_detec = fig_final.add_subplot(20,1,1)
det_y = np.asarray(np.ones((1,len(loc_data['time_min_offset']))))[0]
det_x = np.asarray(loc_data['time_min_offset'])
ax_detec.scatter(det_x,det_y, c=loc_data['time_min_offset'],cmap=cmap,norm=norm,s=12)
ax_detec.set_xlim(ax_spectro.get_xlim())
ax_detec.get_xaxis().set_visible(False)
ax_detec.get_yaxis().set_visible(False)
ax_detec.axis('off')

# #pos =[left, bottom, width, height]
# box = ax_detec.get_position()
# box.y0 = box.y0 + 0.6
# box.y1 = box.y1 + 0.6
# ax_detec.set_position(box)

#size = fig_final.get_size_inches()


plt.subplots_adjust(left=0.08, bottom=0.1, right=0.95, top=0.95, wspace=0, hspace=0)


# divider2 = make_axes_locatable(ax_spectro)
# cax2 = divider2.append_axes('top', size=0.2, pad=10.0)
# det_y = np.asarray(np.ones((1,len(loc_data['time_min_offset']))))[0]
# det_x = np.asarray(loc_data['time_min_offset'])
# cax2.plot(det_x,det_y,'.r')
# cax2.set_xlim(ax_spectro.get_xlim())


# ax_cmap = mpl.colorbar.ColorbarBase(cax, cmap=cmap,
#                                     norm=norm,
#                                     orientation='horizontal')



gs = fig_final.add_gridspec(3,2)

# plot localization top
ax_toploc = fig_final.add_subplot(gs[1:,1])
plot_top_view(hydrophones_config,loc_data,params,cmap,norm, ax_toploc)
ax_toploc.set_anchor('E')

# plot localization side
#ax_sideloc = fig_final.add_subplot(3,3,7,sharex = ax_toploc)
ax_sideloc = fig_final.add_subplot(gs[1:,0])
plot_side_view(hydrophones_config,loc_data,params,cmap,norm,ax_sideloc)
ax_sideloc.set_anchor('W')

# set the spacing between subplots
plt.subplots_adjust(wspace=0, hspace=0)


# # plot video frame 1
# fig_video1, ax_video1 = plt.subplots(1,1)
# frame1_sec = 152.8 # second detection -> 16:38:59.8
# #ax_video1 = fig_final.add_subplot(3,3,5)
# plot_video_frame(video_file,frame1_sec, ax_video1)
# ax_video1.get_xaxis().set_visible(False)
# ax_video1.get_yaxis().set_visible(False)

# # plot video frame 2
# fig_video2, ax_video2 = plt.subplots(1,1)
# frame2_sec = 160 # 4th detection -> 16:39:07
# #ax_video2 = fig_final.add_subplot(3,3,6)
# plot_video_frame(video_file,frame2_sec, ax_video2)
# ax_video2.get_xaxis().set_visible(False)
# ax_video2.get_yaxis().set_visible(False)


fig_final.set_size_inches(8.6,6.72)


box = ax_spectro.get_position()
box.y0 = box.y0 - 0.03
box.y1 = box.y1 - 0.03
ax_spectro.set_position(box)









#ax fig.subplots_adjust(bottom=0.5)

# # plot hydrophones
# fig1 = plt.figure()
# ax = fig1.add_subplot(111, projection='3d')
# colors = matplotlib.cm.tab10(hydrophones_config.index.values)

# for index, hp in hydrophones_config.iterrows():
#     point = ax.scatter(hp['x'],hp['y'],hp['z'],
#                     s=20,
#                     #color=colors[index],
#                     color='black',
#                     label=hp['name'],
#                     alpha=0.5,
#                     )


# # plot frame
# ax.plot([-0.95, -0.95],[-0.9, 0.9],[-0.88,-0.88],
#         linewidth=5,
#         alpha=0.2,
#         linestyle= 'solid',
#         color='red',
#         )

# ax.plot([+0.95, +0.95],[-0.9, 0.9],[-0.88,-0.88],
#         linewidth=5,
#         alpha=0.2,
#         linestyle= 'solid',
#         color='red',
#         )

# # plot localizations
# ax.scatter(loc_data['x'], loc_data['y'],loc_data['z'],
#                     s=loc_size,
#                     marker=loc_marker,
#                     color=loc_color,
#                     alpha=loc_alpha,
#                     )
# # plot uncertainties
# for idx, loc_point in loc_data.iterrows():   
#     ax.plot([loc_point['x']-loc_point['x_std'],loc_point['x']+loc_point['x_std']],
#             [loc_point['y'],loc_point['y']],
#             [loc_point['z'],loc_point['z']],             
#             linewidth=uncertainty_width,
#             linestyle=uncertainty_style,
#             color=uncertainty_color,
#             alpha=uncertainty_alpha,
#             )

#     ax.plot([loc_point['x'],loc_point['x']],
#             [loc_point['y']-loc_point['y_std'],loc_point['y']+loc_point['y_std']],
#             [loc_point['z'],loc_point['z']],
#             linewidth=uncertainty_width,
#             linestyle=uncertainty_style,
#             color=uncertainty_color,
#             alpha=uncertainty_alpha,
#             )
    
#     ax.plot([loc_point['x'],loc_point['x']],
#             [loc_point['y'],loc_point['y']],
#             [loc_point['z']-loc_point['z_std'], loc_point['z']+loc_point['z_std']],
#             linewidth=uncertainty_width,
#             linestyle=uncertainty_style,
#             color=uncertainty_color,
#             alpha=uncertainty_alpha,
#             )
# # Axes labels
# ax.set_xlabel('X (m)', labelpad=10)
# ax.set_ylabel('Y (m)', labelpad=10)
# ax.set_zlabel('Z (m)', labelpad=10)

# #ax.view_init(0, 0)
# #ax.view_init(azim=-1, elev=-1)

# # legend
# #ax.legend(bbox_to_anchor=(1.07, 0.7, 0.3, 0.2), loc='upper left')
# plt.tight_layout()
# plt.show()



# -*- coding: utf-8 -*-
"""
Created on Mon Sep 20 14:55:40 2021

@author: xavier.mouy
"""
import os
import sys
sys.path.append(r'C:\Users\xavier.mouy\Documents\GitHub\ecosound') # Adds higher directory to python modules path.

import pandas as pd
import matplotlib.pyplot as plt
import mpl_toolkits.mplot3d
import matplotlib.cm
import matplotlib as mpl
from mpl_toolkits.axes_grid1 import make_axes_locatable
#import scipy.spatial
import numpy as np
import datetime
import math

from ecosound.core.audiotools import Sound, upsample
from ecosound.core.spectrogram import Spectrogram
from ecosound.core.measurement import Measurement
from ecosound.detection.detector_builder import DetectorFactory
from ecosound.visualization.grapher_builder import GrapherFactory
import ecosound.core.tools
from ecosound.core.tools import derivative_1d, envelope, read_yaml
from localizationlib import euclidean_dist, calc_hydrophones_distances, calc_tdoa, defineReceiverPairs, defineJacobian, predict_tdoa, linearized_inversion, solve_iterative_ML, defineCubeVolumeGrid, defineSphereVolumeGrid
import platform
import cv2

def plot_spectrogram(audio_file,loc,t1_sec, t2_sec, geometry=(1,1,1)):
    
    fmin=0
    fmax=1000
    frame= 0.0625
    window_type= 'hann'
    nfft=0.0853
    step=0.01
    channel=0
    chunk=[t1_sec,t2_sec]
    
    graph_spectros = GrapherFactory('SoundPlotter', title='Spectrograms', frequency_max=fmax)
    sound = Sound(audio_file)
    sound.read(channel=channel, chunk=chunk, unit='sec', detrend=True)
    # Calculates  spectrogram
    spectro = Spectrogram(frame, window_type, nfft, step, sound.waveform_sampling_frequency, unit='sec')
    spectro.compute(sound, dB=True, use_dask=False)
    # Crop unused frequencies
    spectro.crop(frequency_min=fmin, frequency_max=fmax, inplace=True)
    # Plot
    graph_spectros.add_data(spectro)    
    
    #graph_spectros.add_annotation(loc, panel=0, color='burlywood',label='Detections')
    graph_spectros.add_annotation(loc, panel=0, color='peachpuff')
    
    graph_spectros.colormap = 'binary' #'jet'
    fig, ax = graph_spectros.show()

    if ax.get_geometry() != geometry :
        ax.change_geometry(*geometry)        
    return fig, ax

def plot_top_view(hydrophones_config,loc_data,params, ax, color='black',frame_on=True):
    
    #fig1 = plt.figure()
    #ax = fig1.add_subplot(111)
    #colors = matplotlib.cm.tab10(hydrophones_config.index.values)
    
    if frame_on:
        # plot hydrophones
        for index, hp in hydrophones_config.iterrows():
            point = ax.scatter(hp['x'],hp['y'],
                               s=20,
                               #color=colors[index],
                               color='gainsboro',
                               edgecolors='dimgray',
                               #markeredgecolor='r',
                               label=hp['name'],
                               alpha=1,
                               zorder=3
                               )
    
        # plot frame
        frame_color = 'whitesmoke'
        frame_alpha = 1
        frame_width = 3
        
        rectangle = plt.Rectangle((-0.1,-0.2), 0.2, 0.4,
                                  linewidth=1,
                                  ec='dimgray',
                                  alpha=frame_alpha,
                                  facecolor='lightgrey')
        
        ax.add_patch(rectangle)
        
        ax.plot([-0.46, 0.48],[0,0],
              linewidth=1 ,
              alpha=frame_alpha,
              linestyle= 'solid',
              color='dimgray',
              )
        ax.plot([0, 0],[0,0.49],
              linewidth=1 ,
              alpha=frame_alpha,
              linestyle= 'solid',
              color='dimgray',
              )
        
    #c=z, cmap=cmap, norm=norm
    # plot localizations
    ax.scatter(loc_data['x'], loc_data['y'],#c=loc_data['time_min_offset'],
                        s=params['loc_size'].values[0],
                        marker=params['loc_marker'].values[0],
                        color=color,
                        alpha=params['loc_alpha'].values[0],
                        #cmap=cmap,
                        #norm=norm,
                        zorder=5,
                        )
    # plot uncertainties
    for idx, loc_point in loc_data.iterrows():   
        ax.plot([loc_point['x_min_CI99'],loc_point['x_max_CI99']],
                [loc_point['y'],loc_point['y']],
                #c=loc_point['time_min_offset'],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                color=color,
                #color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                #cmap=cmap,
                #norm=norm,
                )
    
        ax.plot([loc_point['x'],loc_point['x']],
                [loc_point['y_min_CI99'],loc_point['y_max_CI99']],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                color=color,
                #color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                )
        
    # Axes labels
    ax.set_xlabel('X (m)', labelpad=4)
    ax.set_ylabel('Y (m)', labelpad=4)
    ax.set_xlim(params['x_min'].values[0], params['x_max'].values[0])
    ax.set_ylim(params['y_min'].values[0], params['y_max'].values[0])
    #ax.grid()
    ax.set_aspect('equal', adjustable='box')
    return ax


def plot_side_view(hydrophones_config,loc_data,params,ax, color='black',frame_on=True):
    
    #fig1 = plt.figure()
    #ax = fig1.add_subplot(111)
    #colors = matplotlib.cm.tab10(hydrophones_config.index.values)
    
    # plot hydrophones
    for index, hp in hydrophones_config.iterrows():
        point = ax.scatter(hp['x'],hp['z'],
                           s=20,
                           #color=colors[index],
                           color='gainsboro',
                           edgecolors='dimgray',
                           #markeredgecolor='r',
                           label=hp['name'],
                           alpha=1,
                           zorder=3
                           )

    # plot frame
    frame_color = 'whitesmoke'
    frame_alpha = 1
    frame_width = 3
    
    rectangle = plt.Rectangle((-0.1,-0.1), 0.2, 0.1,
                          linewidth=1,
                          ec='dimgray',
                          alpha=frame_alpha,
                          facecolor='lightgrey')
        
    ax.add_patch(rectangle)
        
    ax.plot([-0.46, 0.48],[0,0],
      linewidth=1 ,
      alpha=frame_alpha,
      linestyle= 'solid',
      color='dimgray',
      )
    
    ax.plot([0, 0],[0,0.54],
      linewidth=1 ,
      alpha=frame_alpha,
      linestyle= 'solid',
      color='dimgray',
      )
    
    # plot localizations
    ax.scatter(loc_data['x'], loc_data['z'],#c=loc_data['time_min_offset'],
                        s=params['loc_size'].values[0],
                        marker=params['loc_marker'].values[0],
                        color=color,
                        alpha=params['loc_alpha'].values[0],
                        #cmap=cmap,
                        #norm=norm,
                        zorder=5
                        )
    # plot uncertainties
    for idx, loc_point in loc_data.iterrows():   
        ax.plot([loc_point['x_min_CI99'],loc_point['x_max_CI99']],
                [loc_point['z'],loc_point['z']],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                color=color,
                #color=params['uncertainty_color'].values[0],
                #color=cmap(norm(loc_point['time_min_offset'])),
                alpha=params['uncertainty_alpha'].values[0],
                )
    
        ax.plot([loc_point['x'],loc_point['x']],
                [loc_point['z_min_CI99'],loc_point['z_max_CI99']],
                linewidth=params['uncertainty_width'].values[0],
                linestyle=params['uncertainty_style'].values[0],
                #color=params['uncertainty_color'].values[0],
                #color=cmap(norm(loc_point['time_min_offset'])),
                color=color,
                alpha=params['uncertainty_alpha'].values[0],
                )
        
    # Axes labels
    ax.set_xlabel('X (m)', labelpad=4)
    ax.set_ylabel('Z (m)', labelpad=4)
    ax.set_xlim(params['x_min'].values[0], params['x_max'].values[0])
    ax.set_ylim(params['z_min'].values[0], params['z_max'].values[0])
    #ax.grid()
    ax.set_aspect('equal', adjustable='box')
    return ax

def plot_video_frame(video_file,frame_time_sec, ax):
    frame = []
    cap = cv2.VideoCapture(video_file)
    if (cap.isOpened()== False):        
        print("Error opening video stream or file")
    else:
     	# Read until video is completed
        idx=0
        while(cap.isOpened()):            
            ret, frame = cap.read()            
            if ret == True:
                idx+=1
                fps = cap.get(cv2.CAP_PROP_FPS)  # OpenCV2 version 2 used "CV_CAP_PROP_FPS"                
                current_time = idx*(1/fps)
                if current_time >= frame_time_sec:
                    frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB) 
                    frame = frame[90:900,340:1220]
                    break                
            # Break the loop
            else:
                break
        cap.release()
        #cv2.destroyAllWindows()
    return ax.imshow(frame)
## ###########################################################################


hp_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\hydrophones_config_HI-201909.csv'
indir=r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\results\mobile_array_ROV'
audio_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\5147.200910210736.wav'
t1_sec = 108#106
t2_sec = 113.5#119


# loc_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\results\large-array_quillback\AMAR173.4.20190920T161248Z.nc'
# audio_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\AMAR173.1.20190920T161248Z.wav'
# video_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\3420_FishCam01_20190920T163627.613206Z_1600x1200_awb-auto_exp-night_fr-10_q-20_sh-0_b-50_c-0_i-400_sat-0.mp4'
# hp_config_file =  r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\hydrophones_config_07-HI.csv'
# t1_sec = 1570
# t2_sec = 1587#1590

filter_x=[-5, 5]
filter_y=[-5, 5]
filter_z=[-2, 5]
filter_x_std=5
filter_y_std=5
filter_z_std=5

params=pd.DataFrame({
    'loc_color': ['black'],
    'loc_marker': ['o'],
    'loc_alpha': [1],
    'loc_size': [5],
    'uncertainty_color': ['black'],
    'uncertainty_style': ['-'],
    'uncertainty_alpha': [1], #0.7
    'uncertainty_width': [0.2], #0.2
    'x_min':[-3],
    'x_max':[3],
    'y_min':[-3],
    'y_max':[3],
    'z_min':[-3],
    'z_max':[3],    
    })
    
## ###########################################################################

## load localization results 0 degrees
file1 = '0_deg.nc'
loc1 = Measurement()
loc1.from_netcdf(os.path.join(indir,file1))
# loc1_data = loc1.data
# # Filter
# loc1_data = loc1_data.dropna(subset=['x', 'y','z']) # remove NaN
# loc1_data = loc1_data.loc[(loc1_data['x']>=min(filter_x)) & 
#                         (loc1_data['x']<=max(filter_x)) &
#                         (loc1_data['y']>=min(filter_y)) & 
#                         (loc1_data['y']<=max(filter_y)) &
#                         (loc1_data['z']>=min(filter_z)) & 
#                         (loc1_data['z']<=max(filter_z)) &
#                         (loc1_data['x_std']<= filter_x_std) & 
#                         (loc1_data['y_std']<= filter_y_std) &
#                         (loc1_data['z_std']<= filter_z_std)
#                         ]

# # Adjust detection times
# loc1_data['time_min_offset'] = loc1_data['time_min_offset'] - t1_sec
# loc1_data['time_max_offset'] = loc1_data['time_max_offset'] - t1_sec

# # update loc object
# loc1.data = loc1_data

# ## load localization results 90 degrees
# file2 = '90_degrees.nc'
# loc2 = Measurement()
# loc2.from_netcdf(os.path.join(indir,file2))
# loc2_data = loc2.data
# # Filter
# loc2_data = loc2_data.dropna(subset=['x', 'y','z']) # remove NaN
# loc2_data = loc2_data.loc[(loc2_data['x']>=min(filter_x)) & 
#                         (loc2_data['x']<=max(filter_x)) &
#                         (loc2_data['y']>=min(filter_y)) & 
#                         (loc2_data['y']<=max(filter_y)) &
#                         (loc2_data['z']>=min(filter_z)) & 
#                         (loc2_data['z']<=max(filter_z)) &
#                         (loc2_data['x_std']<= filter_x_std) & 
#                         (loc2_data['y_std']<= filter_y_std) &
#                         (loc2_data['z_std']<= filter_z_std)
#                         ]
# # update loc object
# loc2.data = loc2_data

# ## load localization results 90 degrees
# file3 = '180_degrees.nc'
# loc3 = Measurement()
# loc3.from_netcdf(os.path.join(indir,file3))
# loc3_data = loc3.data
# # Filter
# loc3_data = loc3_data.dropna(subset=['x', 'y','z']) # remove NaN
# loc3_data = loc3_data.loc[(loc3_data['x']>=min(filter_x)) & 
#                         (loc3_data['x']<=max(filter_x)) &
#                         (loc3_data['y']>=min(filter_y)) & 
#                         (loc3_data['y']<=max(filter_y)) &
#                         (loc3_data['z']>=min(filter_z)) & 
#                         (loc3_data['z']<=max(filter_z)) &
#                         (loc3_data['x_std']<= filter_x_std) & 
#                         (loc3_data['y_std']<= filter_y_std) &
#                         (loc3_data['z_std']<= filter_z_std)
#                         ]
# # update loc object
# loc3.data = loc3_data

# ## load localization results -90 degrees
# file4 = 'minus90_degrees.nc'
# loc4 = Measurement()
# loc4.from_netcdf(os.path.join(indir,file4))
# loc4_data = loc4.data
# # Filter
# loc4_data = loc4_data.dropna(subset=['x', 'y','z']) # remove NaN
# loc4_data = loc4_data.loc[(loc4_data['x']>=min(filter_x)) & 
#                         (loc4_data['x']<=max(filter_x)) &
#                         (loc4_data['y']>=min(filter_y)) & 
#                         (loc4_data['y']<=max(filter_y)) &
#                         (loc4_data['z']>=min(filter_z)) & 
#                         (loc4_data['z']<=max(filter_z)) &
#                         (loc4_data['x_std']<= filter_x_std) & 
#                         (loc4_data['y_std']<= filter_y_std) &
#                         (loc4_data['z_std']<= filter_z_std)
#                         ]
# # update loc object
# loc4.data = loc4_data

df = pd.read_csv(r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\results\mobile_array_ROV\localizations_matlab_with_CI.csv')
loc1_data = df.loc[0:7] # 0 degrees
loc3_data = df.loc[8:14]
loc2_data = df.loc[15:19]
loc4_data = df.loc[20:27]

## load hydrophone locations
hydrophones_config = pd.read_csv(hp_config_file)


# Plot spectrogram
fig_final, ax_spectro = plot_spectrogram(audio_file,loc1,t1_sec, t2_sec, geometry=(5,1,1))    
ax_spectro.set_title("")


plt.subplots_adjust(left=0.08, bottom=0.1, right=0.95, top=0.95, wspace=0, hspace=0)


gs = fig_final.add_gridspec(3,2)

# plot localization top
ax_toploc = fig_final.add_subplot(gs[1:,1])
plot_top_view(hydrophones_config,loc1_data,params, ax_toploc,color='firebrick',frame_on=True)
plot_top_view(hydrophones_config,loc2_data,params, ax_toploc,color='seagreen',frame_on=False)
plot_top_view(hydrophones_config,loc3_data,params, ax_toploc,color='darkorange',frame_on=False)
plot_top_view(hydrophones_config,loc4_data,params, ax_toploc,color='darkblue',frame_on=False)
ax_toploc.set_anchor('E')

# # plot localization side
ax_sideloc = fig_final.add_subplot(gs[1:,0])
plot_side_view(hydrophones_config,loc1_data,params,ax_sideloc,color='firebrick',frame_on=True)
plot_side_view(hydrophones_config,loc2_data,params, ax_sideloc,color='seagreen',frame_on=False)
plot_side_view(hydrophones_config,loc3_data,params, ax_sideloc,color='darkorange',frame_on=False)
plot_side_view(hydrophones_config,loc4_data,params, ax_sideloc,color='darkblue',frame_on=False)


ax_sideloc.set_anchor('W')

# set the spacing between subplots
plt.subplots_adjust(wspace=0, hspace=0)



fig_final.set_size_inches(8.6,6.72)


box = ax_spectro.get_position()
box.y0 = box.y0 - 0.03
box.y1 = box.y1 - 0.03
ax_spectro.set_position(box)









#ax fig.subplots_adjust(bottom=0.5)

# # plot hydrophones
# fig1 = plt.figure()
# ax = fig1.add_subplot(111, projection='3d')
# colors = matplotlib.cm.tab10(hydrophones_config.index.values)

# for index, hp in hydrophones_config.iterrows():
#     point = ax.scatter(hp['x'],hp['y'],hp['z'],
#                     s=20,
#                     #color=colors[index],
#                     color='black',
#                     label=hp['name'],
#                     alpha=0.5,
#                     )


# # plot frame
# ax.plot([-0.95, -0.95],[-0.9, 0.9],[-0.88,-0.88],
#         linewidth=5,
#         alpha=0.2,
#         linestyle= 'solid',
#         color='red',
#         )

# ax.plot([+0.95, +0.95],[-0.9, 0.9],[-0.88,-0.88],
#         linewidth=5,
#         alpha=0.2,
#         linestyle= 'solid',
#         color='red',
#         )

# # plot localizations
# ax.scatter(loc_data['x'], loc_data['y'],loc_data['z'],
#                     s=loc_size,
#                     marker=loc_marker,
#                     color=loc_color,
#                     alpha=loc_alpha,
#                     )
# # plot uncertainties
# for idx, loc_point in loc_data.iterrows():   
#     ax.plot([loc_point['x']-loc_point['x_std'],loc_point['x']+loc_point['x_std']],
#             [loc_point['y'],loc_point['y']],
#             [loc_point['z'],loc_point['z']],             
#             linewidth=uncertainty_width,
#             linestyle=uncertainty_style,
#             color=uncertainty_color,
#             alpha=uncertainty_alpha,
#             )

#     ax.plot([loc_point['x'],loc_point['x']],
#             [loc_point['y']-loc_point['y_std'],loc_point['y']+loc_point['y_std']],
#             [loc_point['z'],loc_point['z']],
#             linewidth=uncertainty_width,
#             linestyle=uncertainty_style,
#             color=uncertainty_color,
#             alpha=uncertainty_alpha,
#             )
    
#     ax.plot([loc_point['x'],loc_point['x']],
#             [loc_point['y'],loc_point['y']],
#             [loc_point['z']-loc_point['z_std'], loc_point['z']+loc_point['z_std']],
#             linewidth=uncertainty_width,
#             linestyle=uncertainty_style,
#             color=uncertainty_color,
#             alpha=uncertainty_alpha,
#             )
# # Axes labels
# ax.set_xlabel('X (m)', labelpad=10)
# ax.set_ylabel('Y (m)', labelpad=10)
# ax.set_zlabel('Z (m)', labelpad=10)

# #ax.view_init(0, 0)
# #ax.view_init(azim=-1, elev=-1)

# # legend
# #ax.legend(bbox_to_anchor=(1.07, 0.7, 0.3, 0.2), loc='upper left')
# plt.tight_layout()
# plt.show()



# -*- coding: utf-8 -*-
"""
Created on Thu Sep 23 16:08:18 2021

@author: xavier.mouy
"""
import matplotlib.pyplot as plt
import numpy as np

def func(f):
    x = np.linspace(-2, 2, 200)
    duration = 2
    fig, ax = plt.subplots()
    ax.plot(x, np.sin(x*2*np.pi*freq[90]))
    return mplfig_to_npimage(fig)

freq = np.linspace(0, 1, 100)

# creating animation
duration = 2
animation = VideoClip(func, duration = duration)

#animation.ipython_display(fps = 20, loop = True, autoplay = True)



 
# # importing movie py libraries
# from moviepy.editor import VideoClip
# from moviepy.video.io.bindings import mplfig_to_npimage
 
# # numpy array
# x = np.linspace(-2, 2, 200)
 
# # duration of the video
# duration = 2
 
# # matplot subplot
# fig, ax = plt.subplots()
 
# # method to get frames
# def make_frame(t):
     
#     # clear
#     ax.clear()
     
#     # plotting line
#     ax.plot(x, np.sinc(x**2) + np.sin(x + 2 * np.pi / duration * t), lw = 3)
#     ax.set_ylim(-1.5, 2.5)
     
#     # returning mumpy image
#     return mplfig_to_npimage(fig)
 
# # creating animation
# animation = VideoClip(make_frame, duration = duration)
 
# # displaying animation with auto play and looping
# animation.ipython_display(fps = 20, loop = True, autoplay = True)


# -*- coding: utf-8 -*-
"""
Created on Wed Apr  7 10:07:53 2021

@author: xavier.mouy
"""
import os
import sys
sys.path.append(r'C:\Users\xavier.mouy\Documents\GitHub\ecosound') # Adds higher directory to python modules path.

import pandas as pd
import matplotlib.pyplot as plt
import mpl_toolkits.mplot3d
import matplotlib.cm
#import scipy.spatial
import numpy as np
import datetime
import math

from ecosound.core.audiotools import Sound, upsample
from ecosound.core.spectrogram import Spectrogram
from ecosound.core.measurement import Measurement
from ecosound.detection.detector_builder import DetectorFactory
from ecosound.visualization.grapher_builder import GrapherFactory
import ecosound.core.tools
from ecosound.core.tools import derivative_1d, envelope, read_yaml
from localizationlib import euclidean_dist, calc_hydrophones_distances, calc_tdoa, defineReceiverPairs, defineJacobian, predict_tdoa, linearized_inversion, solve_iterative_ML, defineCubeVolumeGrid, defineSphereVolumeGrid,defineSphereSurfaceGrid
import platform

def find_audio_files(filename, hydrophones_config):
    """ Find corresponding files and channels for all the hydrophones of the array """
    filename = os.path.basename(infile)
    # Define file tail
    for file_root in hydrophones_config['file_name_root']:
        idx = filename.find(file_root)
        if idx >= 0:
            file_tail = filename[len(file_root):]
            break
    # Loop through channels and define all files paths and audio channels
    audio_files = {'path':[], 'channel':[]}
    for row_idx, row_data in hydrophones_config.iterrows():
        file_path = os.path.join(row_data.data_path, row_data.file_name_root + file_tail)
        chan = row_data.audio_file_channel
        audio_files['path'].append(file_path)
        audio_files['channel'].append(chan)
    return audio_files

def run_detector(infile, channel, config, chunk=None, deployment_file=None):

    sound = Sound(infile)
    # load audio data
    if chunk:
        sound.read(channel=channel, chunk=[t1, t2], unit='sec', detrend=True)
        time_offset_sec = t1
    else:
        sound.read(channel=channel, detrend=True)
        time_offset_sec = 0

    # Calculates  spectrogram
    spectro = Spectrogram(config['SPECTROGRAM']['frame_sec'],
                                  config['SPECTROGRAM']['window_type'],
                                  config['SPECTROGRAM']['nfft_sec'],
                                  config['SPECTROGRAM']['step_sec'],
                                  sound.waveform_sampling_frequency,
                                  unit='sec',)
    spectro.compute(sound,
                    config['SPECTROGRAM']['dB'],
                    config['SPECTROGRAM']['use_dask'],
                    config['SPECTROGRAM']['dask_chunks'],)

    spectro.crop(frequency_min=config['SPECTROGRAM']['fmin_hz'],
                         frequency_max=config['SPECTROGRAM']['fmax_hz'],
                         inplace=True,
                         )
    # Denoise
    spectro.denoise(config['DENOISER']['denoiser_name'],
                    window_duration=config['DENOISER']['window_duration_sec'],
                    use_dask=config['DENOISER']['use_dask'],
                    dask_chunks=tuple(config['DENOISER']['dask_chunks']),
                    inplace=True)
    # Detector
    file_timestamp = ecosound.core.tools.filename_to_datetime(sound.file_full_path)[0]
    detector = DetectorFactory(config['DETECTOR']['detector_name'],
                               kernel_duration=config['DETECTOR']['kernel_duration_sec'],
                               kernel_bandwidth=config['DETECTOR']['kernel_bandwidth_hz'],
                               threshold=config['DETECTOR']['threshold'],
                               duration_min=config['DETECTOR']['duration_min_sec'],
                               bandwidth_min=config['DETECTOR']['bandwidth_min_hz']
                               )
    start_time = file_timestamp + datetime.timedelta(seconds=time_offset_sec)
    detections = detector.run(spectro,
                              start_time=start_time,
                              use_dask=config['DETECTOR']['use_dask'],
                              dask_chunks=tuple(config['DETECTOR']['dask_chunks']),
                              debug=False,
                              )
    # add time offset in only a section of recording was analysed.
    detections.data['time_min_offset'] = detections.data['time_min_offset'] + time_offset_sec
    detections.data['time_max_offset'] = detections.data['time_max_offset'] + time_offset_sec

    # add deployment metadata
    detections.insert_metadata(deployment_file, channel=channel)

    # Add file informations
    file_name = os.path.splitext(os.path.basename(sound.file_full_path))[0]
    file_dir = os.path.dirname(sound.file_full_path)
    file_ext = os.path.splitext(sound.file_full_path)[1]
    detections.insert_values(operator_name=platform.uname().node,
                               audio_file_name=file_name,
                               audio_file_dir=file_dir,
                               audio_file_extension=file_ext,
                               audio_file_start_date=ecosound.core.tools.filename_to_datetime(sound.file_full_path)[0]
                               )

    return detections


def plot_data(audio_files,frame, window_type, nfft, step, fmin, fmax, chunk = None, detections=None, detections_channel=0):
    graph_spectros = GrapherFactory('SoundPlotter', title='Spectrograms', frequency_max=fmax)
    graph_waveforms = GrapherFactory('SoundPlotter', title='Waveforms')
    for audio_file, channel in zip(audio_files['path'], audio_files['channel'] ): # for each channel
        # load waveform
        sound = Sound(audio_file)
        sound.read(channel=channel, chunk=chunk, unit='sec', detrend=True)
        # Calculates  spectrogram
        spectro = Spectrogram(frame, window_type, nfft, step, sound.waveform_sampling_frequency, unit='sec')
        spectro.compute(sound, dB=True, use_dask=False)
        # Crop unused frequencies
        spectro.crop(frequency_min=fmin, frequency_max=fmax, inplace=True)
        # Plot
        graph_spectros.add_data(spectro, time_offset_sec=chunk[0])
        graph_waveforms.add_data(sound, time_offset_sec=chunk[0])

    graph_spectros.colormap = 'binary'
    if detections:
        graph_spectros.add_annotation(detections, panel=detections_channel, color='green',label='Detections')
        graph_waveforms.add_annotation(detections, panel=detections_channel, color='green',label='Detections')

    if chunk:
        graph_spectros.time_min = chunk[0]
        graph_spectros.time_max = chunk[1]
        graph_waveforms.time_min = chunk[0]
        graph_waveforms.time_max = chunk[1]

    graph_spectros.show()
    graph_waveforms.show()


def stack_waveforms(audio_files, detec, TDOA_max_sec):
    waveform_stack = []
    for audio_file, channel in zip(audio_files['path'], audio_files['channel'] ): # for each channel
        # load waveform
        chan_wav = Sound(audio_file)
        chan_wav.read(channel=channel,
                      chunk=[detec['time_min_offset']-TDOA_max_sec, detec['time_max_offset']+TDOA_max_sec],
                      unit='sec',
                      detrend=True)
        # bandpass filter
        chan_wav.filter('bandpass', [detec['frequency_min'], detec['frequency_max']])
        # stack
        waveform_stack.append(chan_wav.waveform)
    return waveform_stack


def calc_data_error(tdoa_sec, m, sound_speed_mps,hydrophones_config, hydrophone_pairs):
    """ Calculates tdoa measurement errors. Eq. (9) in Mouy et al. 2018"""
    tdoa_m = predict_tdoa(m, sound_speed_mps, hydrophones_config, hydrophone_pairs)
    Q = len(m)
    M = m.size # number of dimensions of the model (here: X, Y, and Z)
    N = len(tdoa_sec) # number of measurements
    if N > M:
        error_std = np.sqrt((1/(Q*(N-M))) * (sum((tdoa_sec-tdoa_m)**2)))
    else:
        error_std = np.sqrt((sum((tdoa_sec-tdoa_m)**2)))
    return error_std


def calc_loc_errors(tdoa_errors_std, m, sound_speed_mps, hydrophones_config, hydrophone_pairs):
    """ Calculates localization errors. Eq. (8) in Mouy et al. 2018."""
    A = defineJacobian(hydrophones_config, m, sound_speed_mps, hydrophone_pairs)
    Cm = (tdoa_errors_std**2) * np.linalg.inv(np.dot(A.transpose(),A)) # Model covariance matrix for IID
    err_std = np.sqrt(np.diag(Cm))
    return pd.DataFrame({'x_std': [err_std[0]], 'y_std': [err_std[1]], 'z_std': [err_std[2]]})

def cartesian2spherical(x,y,z):
    # Converting cartesian to polar coordinate
    const = 180/np.pi
    XsqPlusYsq = x**2 + y**2
    r = math.sqrt(XsqPlusYsq + z**2)               # r
    elev = math.atan2(math.sqrt(XsqPlusYsq),z)     # theta
    az = math.atan2(y,x)*180/np.pi
    return r, az, elev

# # Calculating radius
# radius = math.sqrt( x * x + y * y )
# # Calculating angle (theta) in radian
# theta = math.atan(y/x)
# # Converting theta from radian to degree
# theta = 180 * theta/math.pi

## ############################################################################

#outdir=r'C:\Users\xavier.mouy\Documents\GitHub\ecosound\ecosound\localization'
## ------------------------- LARGE ARRAY --------------------------------------

# # Config files XAV array hornby - quilback
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\hydrophones_config_07-HI.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array_HF2.yaml'
# #detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_large_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\AMAR173.4.20190920T161248Z.wav'
# t1 = 1560#1570
# t2 = 1590
# #detec_idx_forced= 0 

# # Config files XAV array hornby - lingcod - THIS IS THE GOOD ONE!!!
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\hydrophones_config_07-HI.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array_lingcod.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_large_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\AMAR173.4.20190919T204248Z.wav'
# t1 = 697
# t2 = 700
# #detec_idx_forced= 0 

# # Config files XAV array hornby - lingcod - Sep 17
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\hydrophones_config_07-HI.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array_lingcod.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_large_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\AMAR173.4.20190917T011248Z.wav'
# #t1 = 318
# #t2 = 342
# #t1 = 322
# #t2 = 325

# t1 = 335
# t2 = 337
#detec_idx_forced= 0 



# Config files XAV array hornby - lingcod
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\hydrophones_config_07-HI.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_large_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\AMAR173.4.20190916T204248Z.wav'
# # t1 = 589#1440
# # t2 = 604#1459

# #also try
# t1 = 631
# t2 = 635
# #detec_idx_forced= 6 #26

# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\hydrophones_config_07-HI.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_large_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\AMAR173.4.20190919T194248Z.wav'
# t1 =1119 #944# 1052
# t2 =1121 #950# 1056



# # Config files XAV array hornby - lingcod
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\hydrophones_config_07-HI.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_large_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-09-15_HornbyIsland_AMAR_07-HI\AMAR173.4.20190919T001248Z.wav'
# #t1 = 352
# #t2 = 362
# t1 = 903
# t2 = 910
# detec_idx_forced= 10


# # Config files XAV array Ogden Point - ROV on side of array, in front of fishcam2
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\hydrophones_config_04-OGD.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array_HF.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_large_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\AMAR173.4.20190617T161307Z.wav'
# t1 = 1528
# t2 = 1534
# #detec_idx_forced = 1

# # Config files XAV array Ogden Point - Lingcod
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\hydrophones_config_04-OGD.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_large_array.yaml'

# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\AMAR173.4.20190616T154307Z.wav'
# t1 = 274#330
# t2 = 276#334

# # maybe good one.
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\AMAR173.4.20190616T131307Z.wav'
# t1 = 1489
# t2 = 1492

# # good one (the one used in thesis)
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\AMAR173.4.20190617T151307Z.wav'
# t1 = 222
# t2 = 227

# #ok one
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\AMAR173.4.20190616T171307Z.wav'
# t1 = 999
# t2 = 1005


# # Config files XAV array Ogden Point - lingcod
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\hydrophones_config_04-OGD.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_large_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\large_array\2019-06-15_OgdenPoint_AMAR_04-OGD\AMAR173.4.20190616T154307Z.wav'
# t1 = 329
# t2 = 335
# #detec_idx_forced = 1

## ------------------------- MOBILE ARRAY -------------------------------------

# # Config files mobile array - Projector MCauley Point -> fish - 0 degree
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\hydrophones_config_MCP-20200910.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mobile_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mobile_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\5147.200910210736.wav'
# # projector fish signal
# t1 = 106
# t2 = 119
# #detec_idx_forced=2

# # Config files mobile array - Projector MCauley Point -> fish - 90 degree
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\hydrophones_config_MCP-20200910.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mobile_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mobile_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\5147.200910210736.wav'
# # projector fish signal
# t1 = 769 
# t2 = 782 
# #detec_idx_forced=1


# # Config files mobile array - Projector MCauley Point -> fish - 180 degree
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\hydrophones_config_MCP-20200910.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mobile_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mobile_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\5147.200910210736.wav'
# # projector fish signal
# t1 = 910#928.8
# t2 = 923#940
# #detec_idx_forced = 0


# # Config files mobile array - Projector MCauley Point -> fish - -90 degree
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\hydrophones_config_MCP-20200910.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mobile_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mobile_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\5147.200910210736.wav'
# # projector fish signal
# t1 = 1060#928.8
# t2 = 1073#940
# #detec_idx_forced = 0

# # Config files mobile array - Projector MCauley Point -> fish - 0 degree - source 1 m above
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\hydrophones_config_MCP-20200910.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mobile_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mobile_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2020-09-10_Localization_experiment_projector\5147.200910213736.wav'
# # projector fish signal
# t1 = 893.8
# t2 = 906.5
# detec_idx_forced = 0

# Config files mobile array - Horny Island - Black eye goby 
deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\deployment_info.csv'
hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\hydrophones_config_HI-201909.csv'
detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mobile_array.yaml'
localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mobile_array.yaml'
infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\671404070.190916182406.wav'
t1 = 138.6
t2 = 147.9

# t1 = 205
# t2 = 214
#detec_idx_forced = 10

# # Config files mobile array - Horny Island - Quillback part 1 
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\hydrophones_config_HI-201909.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mobile_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mobile_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\671404070.190918170055.wav'
# t1 = 155
# t2 = 159
# detec_idx_forced = 1

# # Config files mobile array - Horny Island - Quillback part 2 
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\hydrophones_config_HI-201909.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mobile_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mobile_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\671404070.190918170055.wav'
# t1 = 258
# t2 = 262
# detec_idx_forced = 1

# # Config files mobile array - Horny Island - Copper Rockfish
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\hydrophones_config_HI-201909.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mobile_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mobile_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mobile_array\2019-09-14_HornbyIsland_Trident\671404070.190918222812.wav'
# outdir = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\results\mobile_array_copper'
# # t1 = 216
# # t2 = 223

# t1 = 844
# t2 = 895

#detec_idx_forced = 1

## -------------------------- MINI ARRAY --------------------------------------

# # Config files mini array - Mill Bay - ROV facing fishcam 
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\hydrophones_config_05-MILL.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_large_array_HF.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mini_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\671404070.190801165502.wav'
# # # ROV in front of camera
# # t1 = 853.5
# # t2 = 877

# # ROV in front of camera
# t1 = 880
# t2 = 900
# # infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\671404070.190801181002.wav'
# # # ROV on left side of camera
# # t1 = 10
# # t2 = 30

#detec_idx_forced = 8

# # Config files mini array - Mill Bay - Shiner Perch facing fishcam 
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\hydrophones_config_05-MILL.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mini_array_HF.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mini_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\671404070.190730164002.wav'
# t1 = 50
# t2 = 59
# detec_idx_forced = 0


# # Config files mini array - Mill Bay - Rockfish - localiuzed behind
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\hydrophones_config_05-MILL.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mini_array.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mini_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\671404070.190801181002.wav'
# t1 = 325
# t2 = 340
# detec_idx_forced = 9

# # Config files mini array - Mill Bay - Copper RockFish on top of fishcam 
# deployment_info_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\deployment_info.csv'
# hydrophones_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\hydrophones_config_05-MILL.csv'
# detection_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\detection_config_mini_array_copper.yaml'
# localization_config_file = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\config_files\localization_config_mini_array.yaml'
# infile = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\data\mini_array\671404070.190801232502.wav'
# t1 = 696
# t2 = 713
# #detec_idx_forced = 2


# load configuration parameters
hydrophones_config= pd.read_csv(hydrophones_config_file, skipinitialspace=True, dtype={'name': str, 'file_name_root': str}) # load hydrophone coordinates (meters)
detection_config = read_yaml(detection_config_file)
localization_config = read_yaml(localization_config_file)
if localization_config['METHOD']['linearized_inversion']:
    localization_method_name = 'Linearized inversion'
    if localization_config['METHOD']['grid_search']:
        raise ValueError('Only 1 localization method allowed.')
else:
    localization_method_name = 'Grid search'
    if localization_config['METHOD']['grid_search'] == False:
        raise ValueError('At least 1 localization method needs to be defined.')

# Look up data files for all channels
audio_files = find_audio_files(infile, hydrophones_config)

# run detector on selected channel
print('DETECTION')
detections = run_detector(audio_files['path'][detection_config['AUDIO']['channel']],
                          audio_files['channel'][detection_config['AUDIO']['channel']],
                          detection_config,
                          chunk = [t1, t2],
                          deployment_file=deployment_info_file)
#detections.insert_values(frequency_min=60)

print(str(len(detections)) + ' detections')

# plot spectrogram/waveforms of all channels and detections
plot_data(audio_files,
          detection_config['SPECTROGRAM']['frame_sec'],
          detection_config['SPECTROGRAM']['window_type'],
          detection_config['SPECTROGRAM']['nfft_sec'],
          detection_config['SPECTROGRAM']['step_sec'],
          detection_config['SPECTROGRAM']['fmin_hz'],
          detection_config['SPECTROGRAM']['fmax_hz'],
          chunk = [t1, t2],
          detections=detections,
          detections_channel=detection_config['AUDIO']['channel'])


# localization
sound_speed_mps = localization_config['ENVIRONMENT']['sound_speed_mps']
ref_channel = localization_config['TDOA']['ref_channel']

# define search window based on hydrophone separation and sound speed
hydrophones_dist_matrix = calc_hydrophones_distances(hydrophones_config)
TDOA_max_sec = np.max(hydrophones_dist_matrix)/sound_speed_mps

# define hydrophone pairs
hydrophone_pairs = defineReceiverPairs(len(hydrophones_config), ref_receiver=ref_channel)

# pre-compute grid search if needed
if localization_config['METHOD']['grid_search']:
    sources = defineSphereVolumeGrid(
        localization_config['GRIDSEARCH']['spacing_m'],
        localization_config['GRIDSEARCH']['radius_m'],
        origin=localization_config['GRIDSEARCH']['origin'])
    if localization_config['GRIDSEARCH']['min_z']:
        sources = sources.loc[sources['z']>=localization_config['GRIDSEARCH']['min_z']]
        sources = sources.reset_index(drop=True)
    # sources = defineCubeVolumeGrid(0.2, 2, origin=[0, 0, 0])
    # sources = defineSphereSurfaceGrid(
    #     10000,
    #     localization_config['GRIDSEARCH']['radius_m'],
    #     origin=localization_config['GRIDSEARCH']['origin'])
    #sources = defineCubeVolumeGrid(0.2, 2, origin=[0, 0, 0])
    try:
        npzfile = np.load(localization_config['GRIDSEARCH']['stored_tdoas'])
        sources_tdoa = npzfile['sources_tdoa']
        sources_array = npzfile['sources']
        sources['x'] = sources_array[:,0]
        sources['y'] = sources_array[:,1]
        sources['z'] = sources_array[:,2]
        print('Succesully loaded precomputed grid TDOAs from file.')
    except:
        print("Couln't read precomputed TDOAs from file, computing grid TDOAs...")    
        sources_tdoa = np.zeros(shape=(len(hydrophone_pairs),len(sources)))
        for source_idx, source in sources.iterrows():
            sources_tdoa[:,source_idx] = predict_tdoa(source, sound_speed_mps, hydrophones_config, hydrophone_pairs).T
        np.savez(os.path.join(outdir,'tdoa_grid'),sources_tdoa=sources_tdoa,sources=sources)    
    # # Azimuth:
    # theta = np.arctan2(sources['y'].to_numpy(),sources['x'].to_numpy())*(180/np.pi)
    # theta = (((theta+90) % 360)-180) *(-1)        
    # # Elevation:
    # phi = np.arctan2(sources['y'].to_numpy()**2+sources['x'].to_numpy()**2,sources['z'].to_numpy())*(180/np.pi)
    # phi = phi - 90
    # sources['theta'] = theta
    # sources['phi'] = phi

# Define Measurement object for the localization results
# if localization_config['METHOD']['linearized_inversion']:
#     localizations = Measurement()
#     localizations.metadata['measurer_name'] = localization_method_name
#     localizations.metadata['measurer_version'] = '0.1'
#     localizations.metadata['measurements_name'] = [['x', 'y', 'z', 'x_std', 'y_std', 'z_std', 'tdoa_errors_std']]

# if localization_config['METHOD']['grid_search']:
#     localizations = Measurement()
#     localizations.metadata['measurer_name'] = localization_method_name
#     localizations.metadata['measurer_version'] = '0.1'
#     localizations.metadata['measurements_name'] = [['theta', 'phi', 'theta_std', 'phi_std', 'tdoa_errors_std']]

localizations = Measurement()
localizations.metadata['measurer_name'] = localization_method_name
localizations.metadata['measurer_version'] = '0.1'
localizations.metadata['measurements_name'] = [['x', 'y', 'z', 'x_std', 'y_std', 'z_std', 'tdoa_errors_std','tdoa_sec_1','tdoa_sec_2','tdoa_sec_3','tdoa_sec_4','tdoa_sec_5']]


# need to define what output is for grid search


# pick single detection (will use loop after)
print('LOCALIZATION')
for detec_idx, detec in detections.data.iterrows():

    if 'detec_idx_forced' in locals():
        print('Warning: forced to only process detection #', str(detec_idx_forced))
        detec = detections.data.iloc[detec_idx_forced]
    
    print( str(detec_idx+1) + '/' + str(len(detections)))

    # load data from all channels for that detection
    waveform_stack = stack_waveforms(audio_files, detec, TDOA_max_sec)

    # readjust signal boundaries to only focus on section with most energy 
    percentage_max_energy = 90
    chunk = ecosound.core.tools.tighten_signal_limits_peak(waveform_stack[detection_config['AUDIO']['channel']], percentage_max_energy)
    waveform_stack = [x[chunk[0]:chunk[1]] for x in waveform_stack]

    # calculate TDOAs
    tdoa_sec, corr_val = calc_tdoa(waveform_stack,
                                   hydrophone_pairs,
                                   detec['audio_sampling_frequency'],
                                   TDOA_max_sec=TDOA_max_sec,
                                   upsample_res_sec=localization_config['TDOA']['upsample_res_sec'],
                                   normalize=localization_config['TDOA']['normalize'],
                                   doplot=False,
                                   )

    if localization_config['METHOD']['grid_search']:    
        delta_tdoa = sources_tdoa - tdoa_sec
        delta_tdoa_norm = np.linalg.norm(delta_tdoa, axis=0)
        min_idx = np.argmin(delta_tdoa_norm)
        sources['delta_tdoa'] = delta_tdoa_norm
        #m = sources.loc[sources['delta_tdoa'] == sources['delta_tdoa'].min()]
        m = pd.DataFrame({'x': sources.loc[min_idx]['x'],
                          'y': sources.loc[min_idx]['y'],
                          'z': sources.loc[min_idx]['z']}, index=[0]
                          )
        # # 3D scatter plot
        # fig = plt.figure()
        # ax = fig.add_subplot(111, projection='3d')
        # colors = matplotlib.cm.tab10(hydrophones_config.index.values)      
        # alphas = 0.5
        # for index, hp in hydrophones_config.iterrows():
        #     point = ax.scatter(hp['x'],hp['y'],hp['z'],
        #                     s=40,
        #                     color=colors[index],
        #                     label=hp['name'],
        #                     )
        # ax.scatter(sources['x'],
        #             sources['y'],
        #             sources['z'],
        #             c=sources['delta_tdoa'],
        #             s=2,
        #             alpha=alphas,)
        # # Axes labels
        # ax.set_xlabel('X (m)', labelpad=10)
        # ax.set_ylabel('Y (m)', labelpad=10)
        # ax.set_zlabel('Z (m)', labelpad=10)
        # # legend
        # ax.legend(bbox_to_anchor=(1.07, 0.7, 0.3, 0.2), loc='upper left')
        # plt.tight_layout()
        # plt.show()
    
        # # # 2D scatter plot
        # fig2 = plt.figure()
        # ax2 = fig2.add_subplot(111)
        # ax2.scatter(sources.theta, sources.phi, c=sources.delta_tdoa)
        # ax2.set_xlabel('Azimuth angle theta (degree)', labelpad=10)
        # ax2.set_ylabel('Elevation angle phi (degree)', labelpad=10)
        # # sources.plot.hexbin(x="theta",
        # #                     y="phi",
        # #                     C="delta_tdoa",
        # #                     #reduce_C_function=np.mean,
        # #                     reduce_C_function=np.min,
        # #                     gridsize=80,
        # #                     cmap="viridis")
        
        # # # from scipy.stats import binned_statistic_2d
        # # # import numpy as np
        
        # # # x = sources.theta.values
        # # # y = sources.phi.values
        # # # z = sources.delta_tdoa.values
        
        # # # x_bins = np.linspace(-180, 180, 360)
        # # # y_bins = np.linspace(-90, 90, 180)
        
        # # # ret = binned_statistic_2d(x, y, z, statistic=np.mean, bins=[x_bins, y_bins])
        # # # fig, (ax0, ax1) = plt.subplots(1, 2, figsize=(12, 4))
        # # #ax0.scatter(x, y, c=z)
        # # # #ax1.imshow(ret.statistic.T, origin='bottom', extent=(0, 10, 10, 20))
        # # # ax1.imshow(ret.statistic.T)    
        
        # # # plt.figure()
        # # # xbins = np.arange(-180,185,5)
        # # # ybins = np.arange(-90,95,5)
        # # # hist = sources.delta_tdoa.groupby([pd.cut(sources.theta, bins=xbins), pd.cut(sources.phi, bins=ybins)]).mean().unstack(fill_value=0)
        # # # im = plt.imshow(hist.values)
        # # # plt.xticks(range(len(hist.index)), hist.index)
        # # # plt.yticks(range(len(hist.columns)), hist.columns)
        # # # plt.colorbar(im)
        # # # plt.show()


        # # Bring all detection and localization informations together
        # detec.loc['theta'] = m['theta'].values[0]
        # detec.loc['phi'] = m['phi'].values[0]        
        # detec.loc['theta_std'] = 0
        # detec.loc['phi_std'] = 0        
        # detec.loc['tdoa_errors_std'] = 0
    
    # Lineralized inversion
    if localization_config['METHOD']['linearized_inversion']:
        [m, iterations_logs] = linearized_inversion(tdoa_sec,
                                                    hydrophones_config,
                                                    hydrophone_pairs,
                                                    localization_config['INVERSION'],
                                                    sound_speed_mps,
                                                    doplot=False)
    
    # Estimate uncertainty
    tdoa_errors_std = calc_data_error(tdoa_sec, m, sound_speed_mps,hydrophones_config, hydrophone_pairs)
    loc_errors_std = calc_loc_errors(tdoa_errors_std, m, sound_speed_mps, hydrophones_config, hydrophone_pairs)

    # Bring all detection and localization informations together
    detec.loc['x'] = m['x'].values[0]
    detec.loc['y'] = m['y'].values[0]
    detec.loc['z'] = m['z'].values[0]
    detec.loc['x_std'] = loc_errors_std['x_std'].values[0]
    detec.loc['y_std'] = loc_errors_std['y_std'].values[0]
    detec.loc['z_std'] = loc_errors_std['z_std'].values[0]
    detec.loc['tdoa_errors_std'] = tdoa_errors_std[0]
    if len(tdoa_sec) >= 3:
        detec.loc['tdoa_sec_1'] = tdoa_sec[0][0]
        detec.loc['tdoa_sec_2'] = tdoa_sec[1][0]
        detec.loc['tdoa_sec_3'] = tdoa_sec[2][0]
    if len(tdoa_sec) > 3:
        detec.loc['tdoa_sec_4'] = tdoa_sec[3][0]
        detec.loc['tdoa_sec_5'] = tdoa_sec[4][0]

    # stack to results into localization object
    localizations.data = localizations.data.append(detec, ignore_index=True)

# Plot hydrophones
fig1 = plt.figure()
ax = fig1.add_subplot(111, projection='3d')
colors = matplotlib.cm.tab10(hydrophones_config.index.values)
# Sources
for index, hp in hydrophones_config.iterrows():
    point = ax.scatter(hp['x'],hp['y'],hp['z'],
                    s=20,
                    color=colors[index],
                    label=hp['name'],
                    )

localization = ax.scatter(localizations.data['x'],
                          localizations.data['y'],
                          localizations.data['z'],
                    s=30,
                    marker='*',
                    color='black',
                    label='Localizations',
                    )

# Axes labels
ax.set_xlabel('X (m)', labelpad=10)
ax.set_ylabel('Y (m)', labelpad=10)
ax.set_zlabel('Z (m)', labelpad=10)
# legend
ax.legend(bbox_to_anchor=(1.07, 0.7, 0.3, 0.2), loc='upper left')
plt.tight_layout()
plt.show()



# -*- coding: utf-8 -*-
"""
Spyder Editor

This is a temporary script file.
"""

# -*- coding: utf-8 -*-
"""
Created on Tue Jul 31 14:33:54 2018

@author: xavier.mouy
"""

import matplotlib.pyplot as plt
import matplotlib.cm as cm
import matplotlib.colors as colors
import mpl_toolkits.mplot3d
import numpy as np
import pickle
import pandas as pd
import localizationlib as loclib

def getArrayUncertainties(R,radius,spacing,V,NoiseSTD,contoursValues):
    
    # Virtual sources coordinates -> Cube of points (Cartesian coordinates)
    vec = np.arange(-radius, radius+spacing, spacing)
    X, Y, Z = np.meshgrid(vec, vec, vec, indexing='ij')
    Sx = np.reshape(X, X.shape[0]*X.shape[1]*X.shape[2])
    Sy = np.reshape(Y, Y.shape[0]*Y.shape[1]*Y.shape[2])
    Sz = np.reshape(Z, Z.shape[0]*Z.shape[1]*Z.shape[2])
    S = pd.DataFrame({'x': Sx, 'y': Sy, 'z': Sz})
    # find location of slice
    ind = np.argmin(abs(vec))
    sliceValue = vec[ind]
    # Nb of receivers    
    nReceivers = R.shape[0]
    
    # Variance of TDOA measurement errors
    NoiseVariance = NoiseSTD**2
    
    # Define receiver pairs for TDOAs
    Rpairs = loclib.defineReceiverPairs (nReceivers)

    # Get list of Jacobian matrice for each source
    J = loclib.defineJacobian(R, S, V, Rpairs) 
    
    # Calculates localization uncertainty for each source
    Uncertainties = loclib.getUncertainties(J, NoiseVariance)   

    # Plots unceratinties of optimized array
    loclib.plotArrayUncertainties(R, S, Uncertainties)

    # PLot hydrophone locations
    f0 = plt.figure()
    ax0 = f0.add_subplot(111, projection='3d')
    ax0.scatter(R['x'], R['y'], R['z'], s=30, c='black')
    ax0.set_xlabel('X (m)', labelpad=10)
    ax0.set_ylabel('Y (m)', labelpad=10)
    ax0.set_zlabel('Z (m)', labelpad=10)
    plt.show()

    # Define and plot plane slices
    f, (ax1, ax2, ax3) = plt.subplots(1, 3, sharey=False,figsize=(16, 5))
    ## XY plane
    XY=np.zeros([len(vec),len(vec)])
    for i in range(len(vec)):
        for jj in range(len(vec)):
            idx = S.index[(S['x']==vec[i]) & (S['y']==vec[jj]) & (S['z']==sliceValue)][0]
            XY[i, jj] = Uncertainties['rms'][idx]              
    CS_XY = ax1.contour(vec, vec, XY, levels=contoursValues, colors=['k'])
    # Receivers
    ax1.plot(R['x'], R['y'], 'go')
    ax1.set_xlabel('X(m)')
    ax1.set_ylabel('Y(m)')
    ax1.grid(True)
    im=ax1.imshow(XY, interpolation='bilinear', origin='lower',
                    cmap=cm.jet, extent=(-radius, radius, -radius, radius),norm=colors.Normalize(vmin = 0, vmax = 20))
    ax1.set_aspect('auto')
    cbar = f.colorbar(im, ax=ax1)
    cbar.ax.set_ylabel('Uncertainty (m)')
    
    ## XZ plane
    XZ=np.zeros([len(vec),len(vec)])
    for i in range(len(vec)):
        for jj in range(len(vec)):
            idx = S.index[(S['x']==vec[i]) & (S['z']==vec[jj]) & (S['y']==sliceValue)][0]
            XZ[i, jj] = Uncertainties['rms'][idx]    
    CS_XZ = ax2.contour(vec, vec, XZ, levels=contoursValues, colors=['k'])
    # Receivers
    ax2.plot(R['x'], R['z'], 'go')
    ax2.set_xlabel('X(m)')
    ax2.set_ylabel('Z(m)')
    ax2.grid(True)
    im=ax2.imshow(XZ, interpolation='bilinear', origin='lower',
                    cmap=cm.jet, extent=(-radius, radius, -radius, radius),norm=colors.Normalize(vmin = 0, vmax = 20))
    ax2.set_aspect('auto')
    cbar = f.colorbar(im, ax=ax2)
    cbar.ax.set_ylabel('Uncertainty (m)')
    
    ## YZ plane
    YZ=np.zeros([len(vec),len(vec)])
    for i in range(len(vec)):
        for jj in range(len(vec)):
            idx = S.index[(S['y']==vec[i]) & (S['z']==vec[jj]) & (S['x']==sliceValue)][0]
            YZ[i, jj] = Uncertainties['rms'][idx]
    CS_YZ = ax3.contour(vec, vec, YZ, levels=contoursValues, colors=['k'])
    # Receivers
    ax3.plot(R['y'], R['z'], 'go')
    ax3.set_xlabel('Y(m)')
    ax3.set_ylabel('Z(m)')
    ax3.grid(True)
    im = ax3.imshow(YZ, interpolation='bilinear', origin='lower',
                    cmap='jet', extent=(-radius, radius, -radius, radius),norm=colors.Normalize(vmin = 0, vmax = 20))
    cbar = f.colorbar(im, ax=ax3)
    cbar.ax.set_ylabel('Uncertainty (m)')
    
#    from mpl_toolkits.axes_grid1 import make_axes_locatable
#    divider = make_axes_locatable(plt.gca())
#    cax = divider.append_axes("right", "5%", pad="3%")
#    plt.colorbar(im, cax=cax)

    #plt.colorbar(im,ax=ax3)
    ax3.set_aspect('auto')
    #plt.tight_layout()
    plt.show()
    #plt.tight_layout()

        
    # Extract contours lines
    coord_CS_XY=[]
    coord_CS_XZ=[]
    coord_CS_YZ=[]
    for i in range(len(contoursValues)):
        p1 = CS_XY.collections[i].get_paths()[0]
        coord_CS_XY.append(p1.vertices)
        p2 = CS_XZ.collections[i].get_paths()[0]
        coord_CS_XZ.append(p2.vertices)
        p3 = CS_YZ.collections[i].get_paths()[0]
        coord_CS_YZ.append(p3.vertices)
    
    return coord_CS_XY, coord_CS_XZ, coord_CS_YZ

## Grid/uncertainties parameters
radius = 3 # meters
spacing = 0.2 # meters 0.3
V = 1488 # Sound speed (m/s)
NoiseSTD = 3.1717e-5 # standard deviation of TDOAs
contoursValues = [0.1,0.2,0.4,0.5]
Colors=['k','r','b','g']

## Hydrophone coordinates (m)
#x=[-0.51,  0.095,  2.430, -0.53,  2.41] Jen hydrophones
#y=[-0.10, -0.125, -0.115, -0.12, -0.10]
#z=[-1.82,  0.570, -0.050, -0.05, -1.82]

x=[-0.42,0.03,0,0.47]
y=[0,0.19,0.51,0]
z=[0,0.51,0,0]
R1 = pd.DataFrame({'x':x,'y':y, 'z':z})

## Runs the processing:
XY1, XZ1, YZ1 = getArrayUncertainties(R1,radius,spacing,V,NoiseSTD,contoursValues)




# -*- coding: utf-8 -*-
"""
Created on Thu Sep 30 16:00:26 2021

@author: xavier.mouy
"""

import os
import sys
sys.path.append(r'C:\Users\xavier.mouy\Documents\GitHub\ecosound') # Adds higher directory to python modules path.

import pandas as pd
import matplotlib.pyplot as plt
import mpl_toolkits.mplot3d
import matplotlib.cm
import matplotlib as mpl
from mpl_toolkits.axes_grid1 import make_axes_locatable
#import scipy.spatial
import numpy as np
import datetime
import math

from ecosound.core.audiotools import Sound, upsample
from ecosound.core.spectrogram import Spectrogram
from ecosound.core.measurement import Measurement
from ecosound.detection.detector_builder import DetectorFactory
from ecosound.visualization.grapher_builder import GrapherFactory
import ecosound.core.tools
from ecosound.core.tools import derivative_1d, envelope, read_yaml
from localizationlib import euclidean_dist, calc_hydrophones_distances, calc_tdoa, defineReceiverPairs, defineJacobian, predict_tdoa, linearized_inversion, solve_iterative_ML, defineCubeVolumeGrid, defineSphereVolumeGrid
import platform
import cv2
import xarray


indir = r'C:\Users\xavier.mouy\Documents\Reports_&_Papers\Papers\10-XAVarray_2020\results\mobile_array_copper'
#outfile = r'C:\Users\xavier.mouy\Documents\PhD\Projects\Fish-arrays_stationary\HI07_localizations\restricted_localizations.csv'
filter_x=[-1000, 1000]
filter_y=[-1000, 1000]
filter_z=[-1000, 1000]
filter_x_std=50
filter_y_std=50
filter_z_std=50

# indir = r'C:\Users\xavier.mouy\Documents\PhD\Projects\Fish-arrays_stationary\HI07_localizations\07-HI'
# outfile = r'C:\Users\xavier.mouy\Documents\PhD\Projects\Fish-arrays_stationary\HI07_localizations\restricted_localizations.csv'
# filter_x=[-1.5, 1.5]
# filter_y=[-1.5, 1.5]
# filter_z=[-2, 2]
# filter_x_std=0.3
# filter_y_std=0.3
# filter_z_std=0.3

# load data
print('')
print('Loading dataset')
idx = 0
for infile in os.listdir(indir):
    if infile.endswith(".nc"):
        print(infile)
        
        locs = Measurement()
        locs.from_netcdf(os.path.join(indir,infile))
        loc_data = locs.data
        
        # Filter
        loc_data = loc_data.dropna(subset=['x', 'y','z']) # remove NaN
        loc_data = loc_data.loc[(loc_data['x']>=min(filter_x)) & 
                                (loc_data['x']<=max(filter_x)) &
                                (loc_data['y']>=min(filter_y)) & 
                                (loc_data['y']<=max(filter_y)) &
                                (loc_data['z']>=min(filter_z)) & 
                                (loc_data['z']<=max(filter_z)) &
                                (loc_data['x_std']<= filter_x_std) & 
                                (loc_data['y_std']<= filter_y_std) &
                                (loc_data['z_std']<= filter_z_std)
                                ]
        
        if idx == 0:
            final_df = loc_data
        else:
            final_df = final_df.append(loc_data,ignore_index=True)
        print('s')
        idx +=1
#final_df.to_csv(outfile)